@@ -26,8 +26,9 @@ impl WriterBuilder {
             quote: b'"',
             escape: b'\\',
             double_quote: true,
+            comment: None,
         };
-        WriterBuilder { wtr: wtr }
+        WriterBuilder { wtr }
     }
 
     /// Builder a CSV writer from this configuration.
@@ -54,7 +55,13 @@ impl WriterBuilder {
             Any(b) => {
                 wtr.requires_quotes[b as usize] = true;
             }
-            _ => unreachable!(),
+        }
+        // If the first field of a row starts with a comment character,
+        // it needs to be quoted, or the row will not be readable later.
+        // As requires_quotes is calculated in advance, we force quotes
+        // when a comment character is encountered anywhere in the field.
+        if let Some(comment) = self.wtr.comment {
+            wtr.requires_quotes[comment as usize] = true;
         }
         wtr
     }
@@ -119,6 +126,17 @@ impl WriterBuilder {
         self.wtr.double_quote = yes;
         self
     }
+
+    /// The comment character that will be used when later reading the file.
+    ///
+    /// If `quote_style` is set to `QuoteStyle::Necessary`, a field will
+    /// be quoted if the comment character is detected anywhere in the field.
+    ///
+    /// The default value is None.
+    pub fn comment(&mut self, comment: Option<u8>) -> &mut WriterBuilder {
+        self.wtr.comment = comment;
+        self
+    }
 }
 
 impl Default for WriterBuilder {
@@ -166,23 +184,21 @@ pub struct Writer {
     quote: u8,
     escape: u8,
     double_quote: bool,
+    comment: Option<u8>,
 }
 
 impl Clone for Writer {
     fn clone(&self) -> Writer {
-        let mut requires_quotes = [false; 256];
-        for i in 0..256 {
-            requires_quotes[i] = self.requires_quotes[i];
-        }
         Writer {
             state: self.state.clone(),
-            requires_quotes: requires_quotes,
+            requires_quotes: self.requires_quotes,
             delimiter: self.delimiter,
             term: self.term,
             style: self.style,
             quote: self.quote,
             escape: self.escape,
             double_quote: self.double_quote,
+            comment: self.comment,
         }
     }
 }
@@ -313,7 +329,8 @@ impl Writer {
     /// a field delimiter, then nothing is written to `output`
     /// and `WriteResult::OutputFull` is returned. Otherwise,
     /// `WriteResult::InputEmpty` is returned along with the number of bytes
-    /// written to `output` (which is always `1`).
+    /// written to `output` (which is `1` in case of an unquoted
+    /// field, or `2` in case of an end quote and a field separator).
     pub fn delimiter(
         &mut self,
         mut output: &mut [u8],
@@ -372,9 +389,8 @@ impl Writer {
             self.state.quoting = false;
         }
         let (res, o) = match self.term {
-            Terminator::CRLF => write_pessimistic(&[b'\r', b'\n'], output),
+            Terminator::CRLF => write_pessimistic(b"\r\n", output),
             Terminator::Any(b) => write_pessimistic(&[b], output),
-            _ => unreachable!(),
         };
         if o == 0 {
             return (res, nout);
@@ -424,7 +440,6 @@ impl Writer {
             QuoteStyle::Never => false,
             QuoteStyle::NonNumeric => is_non_numeric(input),
             QuoteStyle::Necessary => self.needs_quotes(input),
-            _ => unreachable!(),
         }
     }
 
@@ -496,7 +511,7 @@ pub fn is_non_numeric(input: &[u8]) -> bool {
     // I suppose this could be faster if we wrote validators of numbers instead
     // of using the actual parser, but that's probably a lot of work for a bit
     // of a niche feature.
-    !s.parse::<f64>().is_ok() && !s.parse::<i128>().is_ok()
+    s.parse::<f64>().is_err() && s.parse::<i128>().is_err()
 }
 
 /// Escape quotes `input` and writes the result to `output`.
@@ -1043,4 +1058,21 @@ mod tests {
         inp = &inp[1..];
         assert_quote!(inp, out, 1, 2, InputEmpty, r#""""#);
     }
+
+    #[test]
+    fn comment_char_is_automatically_quoted() {
+        let mut wtr = WriterBuilder::new().comment(Some(b'#')).build();
+        let out = &mut [0; 1024];
+
+        assert_field!(
+            wtr,
+            b("# abc"),
+            &mut out[..],
+            5,
+            6,
+            InputEmpty,
+            "\"# abc"
+        );
+        assert_write!(wtr, finish, &mut out[..], 1, InputEmpty, "\"");
+    }
 }
